/**
 * Calder - A library for sculpting and manipulating complex 3D structures for
 * the web.
 *
 * Copyright (c) 2018 Paul Bardea, Tammy Liu, Abhishek Madan, Andrew McBurney,
 *                    and Dave Pagurek van Mossel
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/*****************************
 * Armature
 *****************************/

export * from './armature/Animation';
export * from './armature/Armature';
export * from './armature/Constraints';
export * from './armature/CostFunction';
export * from './armature/Generator';
export * from './armature/GuidingVectors';
export * from './armature/Model';
export * from './armature/Node';
export * from './armature/Silhouette';

/*****************************
 * Colors
 *****************************/

export * from './colors/Color';
export * from './colors/CMYKColor';
export * from './colors/RGBColor';

/*****************************
 * Geometry
 *****************************/

export * from './geometry/BakedGeometry';
export * from './geometry/ScalarField';
export * from './geometry/Shape';
export * from './geometry/WorkingGeometry';

/*****************************
 * Interfaces
 *****************************/

export * from './interfaces/Bakeable';

/*****************************
 * Math
 *****************************/

export * from './math/utils';

/*****************************
 * Renderer
 *****************************/

export * from './renderer/Camera';
export * from './renderer/Renderer';
export * from './renderer/Light';
export * from './renderer/Material';
export * from './renderer/commands/createDrawAxes';
export * from './renderer/commands/createDrawObject';
export * from './renderer/commands/createDrawVectorField';
export * from './renderer/commands/createDrawGuidingCurve';

/*****************************
 * Types
 *****************************/

export * from './types/ConstraintTypes';
export * from './types/DebugParams';
export * from './types/ExternalTypes';
export * from './types/RenderParams';
export * from './types/RenderObject';

/*****************************
 * Utils
 *****************************/

export * from './utils/Task';
export * from './utils/quaternion';
export * from './utils/matrix';
